#!/usr/bin/python3
"""
// © University of Southampton IT Innovation Centre, 2018
//
// Copyright in this software belongs to University of Southampton
// IT Innovation Centre of Gamma House, Enterprise Road,
// Chilworth Science Park, Southampton, SO16 7NS, UK.
//
// This software may not be used, sold, licensed, transferred, copied
// or reproduced in whole or in part in any manner or form or in or
// on any media by any person other than in accordance with the terms
// of the Licence Agreement supplied with the software, or otherwise
// without the prior written consent of the copyright owners.
//
// This software is distributed WITHOUT ANY WARRANTY, without even the
// implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
// PURPOSE, except where stated in the Licence Agreement supplied with
// the software.
//
//      Created By :            Nikolay Stanchev
//      Created Date :          02-07-2018
//      Created for Project :   FLAME
"""

from sqlalchemy import Column, String, Integer, UniqueConstraint, and_
from clmcservice.models.meta import Base


class ServiceFunctionEndpoint(Base):
    """
    This class defines the main model of the WHOAMI API, declaring the global tags for a specific service function on a specific endpoint.
    """

    __tablename__ = 'sfendpoint'  # table name in the PostgreSQL database

    __table_args__ = (UniqueConstraint('sf_i', 'sf_endpoint', 'sr'),)  # defines a unique constraint across 3 columns - sf_i, sf_endpoint, sr

    uid = Column(Integer, primary_key=True, autoincrement=True, nullable=False)  # a primary key integer field (auto incremented)

    location = Column(String, nullable=False)  # cluster label
    sfc = Column(String, nullable=False)  # service function chain label
    sfc_i = Column(String, nullable=False)  # service function chain instance identifier
    sf = Column(String, nullable=False)  # service function label
    sf_i = Column(String, nullable=False)   # service function identifier (potentially FQDN)
    sf_endpoint = Column(String, nullable=False)  # service function endpoint (potentially IP address)
    sr = Column(String, nullable=False)  # service router ID - service router that connects the VM to FLAME

    @property
    def json(self):
        """
        Converts an instance of a ServiceFunctionEndpoint to JSON format.

        :return: a python dictionary object
        """

        fields = {c.name: getattr(self, c.name) for c in self.__table__.columns}
        fields.pop("uid")

        return fields

    @staticmethod
    def required_columns():
        """
        Returns the required columns for constructing a valid instance.

        :return: a generator object
        """

        return tuple(column.name for column in ServiceFunctionEndpoint.__table__.columns if column.name != "uid")

    @staticmethod
    def constrained_columns():
        """
        :return: the columns that are uniquely identifying an instance of this model.
        """

        return tuple(column.name for column in ServiceFunctionEndpoint.__table_args__[0].columns)

    @staticmethod
    def get(sf_i, sf_endpoint, sr):
        """
        Gets the instance matching the unique constraint or None if not existing.

        :param sf_i: service function instance
        :param sf_endpoint: service function endpoint
        :param sr: service router

        :return: the first object from the result set that matches the unique constraint or None
        """

        return ServiceFunctionEndpoint.query().filter(and_(ServiceFunctionEndpoint.sf_i == sf_i, ServiceFunctionEndpoint.sf_endpoint == sf_endpoint, ServiceFunctionEndpoint.sr == sr)).first()

    @staticmethod
    def exists(sf_i, sf_endpoint, sr):
        """
        Checks if an instance matching the unique constraint exists.

        :param sf_i: service function instance
        :param sf_endpoint: service function endpoint
        :param sr: service router

        :return: True if exists, False otherwise
        """

        return ServiceFunctionEndpoint.get(sf_i, sf_endpoint, sr) is not None
